//#################################################################################
//	Library to use BMP180 with ATMEL Atmega family
//#################################################################################

#include "bmp180.h"


uint16_t bmp180ReadShort(uint8_t address, uint8_t* error_code)
{
	uint8_t temp[2];
	uint16_t msb=0;
	uint16_t lsb=0;
	uint16_t data;
	
	*error_code |= i2c_start();

	*error_code |= i2c_write(BMP180_SLA, &address, 1);

	*error_code |= i2c_start();
	
	*error_code |= i2c_read(BMP180_SLA, temp, 2);	//  2 ,  

	*error_code |= i2c_stop();

	msb = temp[0];
	lsb = temp[1];

	data = (msb << 8) + lsb;
	return data;
}
//----------------------------------------

uint32_t bmp180ReadLong(uint8_t address, uint8_t* error_code)
{
	uint8_t temp[3];
	uint32_t msb=0;
	uint32_t lsb=0;
	uint32_t xlsb=0;
	uint32_t data;
	
	*error_code |= i2c_start();

	*error_code |= i2c_write(BMP180_SLA, &address, 1);

	*error_code |= i2c_start();

	*error_code |= i2c_read(BMP180_SLA, temp, 3);	//  3 ,  
	
	*error_code |= i2c_stop();

	msb = temp[0];
	lsb = temp[1];
	xlsb = temp[2];

	data = ((msb << 16) + (lsb << 8) + xlsb) >> (8-OSS);
	return data;
}
//----------------------------------------

int32_t bmp180ReadTemp(uint8_t* error_code)
{
	uint8_t temp[2] = {0xF4, 0x2E};
	int32_t temperature = 0;

	*error_code |= i2c_start();
	
	*error_code |= i2c_write(BMP180_SLA, temp, 2);

	*error_code |= i2c_stop();

	_delay_ms(5);	// min time is 4.5ms
	
	temperature = bmp180ReadShort(0xF6,error_code);
		
	return temperature;
}
//----------------------------------------

int32_t bmp180ReadPressure(uint8_t* error_code)
{
	uint8_t temp[2] = {0xF4, 0x34+(OSS<<6)};
	int32_t pressure = 0;

	*error_code |= i2c_start();
	
	*error_code |= i2c_write(BMP180_SLA, temp, 2);

	*error_code |= i2c_stop();

	_delay_ms(25);

	pressure = bmp180ReadLong(0xF6,error_code);
		
	return pressure;
}
//----------------------------------------

void bmp180Convert(int16_t BMP180_calibration_int16_t[],int16_t BMP180_calibration_uint16_t[],int32_t* temperature, int32_t* pressure, uint8_t* error_code)
{

	int8_t i;
	int32_t ut=0;
	int32_t up=0;
	int32_t x1, x2, b5, b6, x3, b3, p;
	uint32_t b4, b7;
	int16_t ac1=BMP180_calibration_int16_t[0];
	int16_t ac2=BMP180_calibration_int16_t[1]; 
	int16_t ac3=BMP180_calibration_int16_t[2]; 
	uint16_t ac4=BMP180_calibration_uint16_t[0];
	uint16_t ac5=BMP180_calibration_uint16_t[1];
	uint16_t ac6=BMP180_calibration_uint16_t[2];
	int16_t b1=BMP180_calibration_int16_t[3]; 
	int16_t b2=BMP180_calibration_int16_t[4];
	//int16_t mb=BMP180_calibration_int16_t[5];
	int16_t mc=BMP180_calibration_int16_t[6];
	int16_t md=BMP180_calibration_int16_t[7];

	if (*error_code==0){
		ut += bmp180ReadTemp(error_code);
	}
	if (*error_code==0){
		up = bmp180ReadPressure(error_code);
	}
	
	x1 = ((int32_t)ut - (int32_t)ac6) * (int32_t)ac5 >> 15;
	x2 = ((int32_t)mc << 11) / (x1 + md);
	b5 = x1 + x2;
	*temperature = (b5 + 8) >> 4;
	
	b6 = b5 - 4000;
	x1 = (b2 * ((b6 * b6) >> 12)) >> 11;
	x2 = (ac2 * b6) >> 11;
	x3 = x1 + x2;
	b3 = (((((int32_t) ac1) * 4 + x3)<<OSS) + 2)>> 2;
	x1 = (ac3 * b6) >> 13;
	x2 = (b1 * ((b6 * b6) >> 12)) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (ac4 * (uint32_t) (x3 + 32768)) >> 15;
	b7 = ((uint32_t) (up - b3) * (50000 >> OSS));
	//p = b7 < 0x80000000 ? (b7 * 2) / b4 : (b7 / b4) * 2;
	
	if (b7 < 0x80000000)
	{
		p = (b7 << 1) / b4;
	}
	else
	{ 
		p = (b7 / b4) << 1;
	}

	x1 = (p >> 8) * (p >> 8);
	x1 = (x1 * 3038) >> 16;
	x2 = (-7357 * p) >> 16;
	*pressure = p + ((x1 + x2 + 3791) >> 4);
}
//----------------------------------------

void BMP180_Calibration(int16_t BMP180_calibration_int16_t[],int16_t BMP180_calibration_uint16_t[], uint8_t* errorcode)
{
	if (*errorcode==0){
		BMP180_calibration_int16_t[0] = bmp180ReadShort(0xAA,errorcode);//ac1
	}	
	if (*errorcode==0){	
		BMP180_calibration_int16_t[1] = bmp180ReadShort(0xAC,errorcode);//ac2
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[2] = bmp180ReadShort(0xAE,errorcode);//ac3
	}
	if (*errorcode==0){	
		BMP180_calibration_uint16_t[0] = bmp180ReadShort(0xB0,errorcode);//ac4
	}
	if (*errorcode==0){	
		BMP180_calibration_uint16_t[1] = bmp180ReadShort(0xB2,errorcode);//ac5
	}
	if (*errorcode==0){	
		BMP180_calibration_uint16_t[2] = bmp180ReadShort(0xB4,errorcode);//ac6
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[3] = bmp180ReadShort(0xB6,errorcode);//b1
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[4] = bmp180ReadShort(0xB8,errorcode);//b2
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[5] = bmp180ReadShort(0xBA,errorcode);//mb
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[6] = bmp180ReadShort(0xBC,errorcode);//mc
	}
	if (*errorcode==0){	
		BMP180_calibration_int16_t[7] = bmp180ReadShort(0xBE,errorcode);//md
	}
}
//----------------------------------------
/*
int32_t bmp180CalcAltitude(int32_t pressure)
{
	float temp;
	int32_t altitude;
	temp = (float) pressure/101325;
	temp = 1-pow(temp, 0.19029);
	//altitude = round(44330*temp*10);
	altitude = 44330*temp*100;
	//get altitude in dm
	return altitude;
}
*/


u08 bmp180_read(void)
{
 	int32_t temperature = 0;
	int32_t pressure = 0;
	int16_t BMP085_calibration_int16_t[8];
	int16_t BMP085_calibration_uint16_t[3];

	bmp180.error = 0;
  
	BMP180_Calibration(BMP085_calibration_int16_t, BMP085_calibration_uint16_t, &bmp180.error);
	
	if (bmp180.error) return bmp180.error;
	
	_delay_ms(10);
	
	bmp180Convert(BMP085_calibration_int16_t, BMP085_calibration_uint16_t, &temperature, &pressure, &bmp180.error);
	
	if (bmp180.error) return bmp180.error;
		
	bmp180.pressure_Pa = pressure;
	bmp180.pressure_mmHg = (float)pressure/133.322;
		
	bmp180.temperature = (float)temperature/10;

	return bmp180.error;
}