/*
 * Created: 2013
 *  Author: Gnitiev V.P.
 *
 * Optimization - O2
 */

//-----Include files----------------------------------------------------------

#include "ds18x20.h"

//-----Defines----------------------------------------------------------------

u08 ds18x20_count;						//     

//-----Headers of function----------------------------------------------------

u08 microlan_rw(u08 send_byte);
u08 do_crc(u08 *data, u08 count);
void  microlan_reset(void);
void  read_device(u08 n_device, u08 *data, u08 count);
void  write_device(u08 n_device, u08 *data, u08 count);

u64 microlan_read_id(void);
void ds18x20_start_convert(void);
void ds18x20_all_read(void);			
u08 microlan_search_id(void);

//-----Begin code-------------------------------------------------------------

void microlan_reset(void)
{
	MICROLAN_PORT &= ~(1 << MICROLAN_BIT);		// 0   

	MICROLAN_DDR |= (1 << MICROLAN_BIT);		// 
	_delay_us(500);
	MICROLAN_DDR &= ~(1 << MICROLAN_BIT);		//  
	_delay_us(500);
}

u08 microlan_rw_bit(u08 send_bit)
{
	u08 read_bit = 0;
		cli();										//       
		MICROLAN_DDR |= (1 << MICROLAN_BIT);		// 
		if ( send_bit ) _delay_us(10);				//  1 - 10 
		else _delay_us(60);							//  0 - 60 
		MICROLAN_DDR &= ~(1 << MICROLAN_BIT);		//  
		_delay_us(5);								//    
		if ( MICROLAN_PIN & (1 << MICROLAN_BIT) ) read_bit = 1; //    1    
		sei();										//   -  
		_delay_us(60);								//   -
	return read_bit;
}

u08 microlan_rw(u08 send_byte)
{
/*****************************************************************************	
	   0
     0   60-120 
******************************************************************************
	   1
     0   1-15 
*****************************************************************************
	 (  0xFF)
     0   1-15 .  ,  
15       0,    0, 
   1,       
*****************************************************************************/
	u08 now_bit = 1, read_byte = 0;

	while (now_bit)
	{
		cli();										//       
		MICROLAN_DDR |= (1 << MICROLAN_BIT);		// 
		if ( send_byte & now_bit ) _delay_us(10);	//  1 - 10 
		else _delay_us(60);							//  0 - 60 
		MICROLAN_DDR &= ~(1 << MICROLAN_BIT);		//  
		_delay_us(5);								//    
		if ( MICROLAN_PIN & (1 << MICROLAN_BIT) ) read_byte |= now_bit; //    1    
		sei();										//   -  
		_delay_us(60);								//   -
		now_bit <<= 1;
	}
	return read_byte;
}

u08 do_crc(u08 *data, u08 count)
{
	u08 now_byte, num=0, now_bit=0, crc=0;
	while (num < count)										//        
	{
		now_byte = *(data+num); // now_byte = data[num];	//   
		now_bit = 0;
		while (now_bit < 8)									//    
		{
			if ((crc & 0x01) ^ ((now_byte >> now_bit) & 0x01)) 	crc = (((crc ^ 0x18) >> 1) | 0x80);
			else												crc = ((crc >> 1) & 0x7F);
			now_bit++;
		}
		num++;
	}
	return crc;	
}

u64 microlan_read_id(void)
{
	u64 id = 0;
	u08 num = 0;
	u08 *_id = &id;

	microlan_reset();						//    .
	microlan_rw(0x33);						//   " " ( 033h).
	while (num<8)
	{
		*(_id + num++) = microlan_rw(0xFF);	//  
	}
	return id;
}

void read_device(u08 n_device, u08 *data, u08 count)
{
/*****************************************************************************
       MicroLAN:

   .
  .
 	  " " ( 055h).
	 8-   ()  .

	  " " ( 0CCh),
  " " ( 044h).

,    (   0.5 ).

   .
  " " ( 055h).
 8-    .
  "  " ( 0BEh).
 9  ( ).
      (  - CRC).
*****************************************************************************/
	u08 *now_byte = &ds18x20_array[n_device].id;			//  8       -    
	u08 num = 0;

	microlan_reset();										//    .
	microlan_rw(0x55);										//   " " ( 055h).

	while (num<8) microlan_rw( *(now_byte+num++) );			//  8-    . (ID + SN + CRC)
	
	microlan_rw(0xBE);										//   "  " ( 0BEh).
	
	num = 0;
	while (num<count) *(data+num++) =  microlan_rw(0xFF);	//  <count>  ( )   data[]
}

void write_device(u08 n_device, u08 *data, u08 count)
{
	u08 *now_byte = &ds18x20_array[n_device].id;		//  8       -    
	u08 num = 0;

	microlan_reset();								//    .
	microlan_rw(0x55);								//   " " ( 055h).

	while (num<8) microlan_rw(*(now_byte+num++));	//  8-    . (ID + SN + CRC)

	num = 0;
	while (num<count) microlan_rw(*(data+num++)); 	//     -  <count>    
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
//		
////////////////////////////////////////////////////////////////////////////////////////////////////////

u08 microlan_search_id(void)
{
	u08 old_end_dual_bit = 0xFF;				//     
	u08 t_b[64];								//    
	u08 n_device = 0;
	u08 dual_bit;
	
	for (u08 i=0; i<64; i++) t_b[i] = 0;
	
	do 
	{
		u08 bit_a, bit_b, bit;
		u64 id = 0;
				
		dual_bit = 0xFF;
		
		microlan_reset();								//    .
		microlan_rw(0xF0);								//   " " ( F0h)

		for (u08 now_bit=0; now_bit<64; now_bit++)
		{
			bit_a = microlan_rw_bit(1);					//   
			bit_b = microlan_rw_bit(1);
	
			if (bit_a & bit_b) return 0;				//  -    
			if (bit_a | bit_b)							//     
			{
				id |= ( (u64)bit_a << now_bit ); 		//  
				microlan_rw_bit(bit_a);					//    -  
			}
		
			else	//      
			{
				if (t_b[now_bit]) 
					bit = 1;					//     (   )  1
				else 
				{
					bit = 0;
			
					if ( (now_bit == old_end_dual_bit) ) 	//              
					{
						bit = 1;							//  1  0
						
						t_b[now_bit] = 1;					//     -      
						for (u08 i=now_bit+1; i<64; i++)	//     
						t_b[i] = 0;
					}	
					else dual_bit = now_bit;				//   -       	
				}					
	
				microlan_rw_bit(bit);						//   
				id |= ( (u64)bit << now_bit );				// +   	
			}//else
		} //for
	
		old_end_dual_bit = dual_bit;
		ds18x20_array[n_device++].id = id; 
		
	} while ( (dual_bit != 0xFF) && (n_device < N_DEVICE_MAX) );
	
	ds18x20_count = n_device;
	return n_device;
}
		

void ds18x20_start_convert(void)
{
	microlan_reset();							//    .
	microlan_rw(0xCC);							//   " " ( 0CCh)
	microlan_rw(0x44);							//   " " ( 044h)   .
}


void ds18x20_all_read(void)
{
	u08 num, temp_err, temp_signarure, temp[9], _temp;

	for (u08 i=0; i < N_DEVICE_MAX; i++) ds18x20_array[i].error = ERR_NO_DEVICE;

	for (u08 i=0; i < ds18x20_count; i++) 					//     
	{
		temp_err = ERR_COMPARE_CRC;							//    CRC,    -  		
		num = 0;
		do {												//         CRC     
		
			read_device(i, temp, 9);						//  9 , 9 - .

			//   
			_temp = 0xFF;
			for (u08 j=0; j<9; j++) _temp &= temp[j];

			if (_temp == 0xFF )
			{
				temp_err = ERR_NO_DEVICE;					//    " "
				break;										//     
			}

			if (temp[8] == do_crc(temp, 8))					//   CRC      
			{
				temp_err = ERR_OK;							//     	

				temp_signarure = ds18x20_array[i].id & 0xFF;//   

					if (temp_signarure == SIGNATURE_DS18B20) 	//   -  ds18b20
					{
						s16 t_temp =   temp[0] | ( temp[1] << 8 );
						ds18x20_array[i].temperature = (float)t_temp / 16; 	//   ds18b20 
						
						break;									//      
					}
					if (temp_signarure == SIGNATURE_DS1820) 	//   -  ds1820
					{
						s16 t_temp =   temp[0] | ( temp[1] << 8 );
						ds18x20_array[i].temperature = (float)t_temp / 2; 	//   ds1820 
						
						break;									//      
					}
				temp_err = ERR_UNKNOW;						//      -    "  "
				break;										//      
			}
			num++;											//   +1
		} while (num < COUNT_TRY_READ);	

		ds18x20_array[i].error = temp_err;
	} // for
}
	
